-- database for vanilla symbol arts + sound definitions
if not (symbolArts)
	local symbolart_sounds = {
		"SA_NORMAL",
		"SA_JOY",
		"SA_ANGER",
		"SA_SORROW",
		"SA_UNEASE",
		"SA_SURPRISED",
		"SA_DOUBT",	// SA_QUESTION
		"SA_HELP",
		"SA_WHISTLE",
		"SA_EMBARRASSMENT",	// SA_SHYNESS
		"SA_NAILEDIT",	// SA_DECISIVE
	}
	for k, v in ipairs(symbolart_sounds)
		local index = k
		if (k > 9) then index = (k == 10) and "A" or "B" end
		
		freeslot("sfx_sasfx"..index)
		rawset(_G, v, sfx_sasfx1 + (k-1))
	end
	
	rawset(_G, "symbolArts", {})
end
local base_SAs = {
	-- srb2 pack
	{name = "Aight.", patch = "SA_AIGHT", sound = SA_NAILEDIT},
	{name = "Amy bruh", patch = "SA_AMYBH", sound = SA_WHISTLE},
	{name = "beedee", patch = "SA_BLMER", sound = SA_UNEASE},
	{name = "Ring Monitor 2.2", patch = "SA_RNGS4", sound = SA_NORMAL},
	{name = "Eggman", patch = "SA_EGGMN", sound = SA_UNEASE},
	{name = "Eggman threaten", patch = "SA_EGGTH", sound = SA_ANGER},
	{name = "Ember peep", patch = "SA_EMBRP", sound = SA_EMBARRASSMENT},
	{name = "FLICKY", patch = "SA_FLCKY"},
	{name = "Hugged", patch = "SA_HUGED", sound = SA_JOY},
	{name = "Knuckles confusion", patch = "SA_KXCON", sound = SA_DOUBT},
	{name = "Missed bubble", patch = "SA_MISDB", sound = SA_WHISTLE},
	{name = "Yeah, I'm a CAST", patch = "SA_MCAST", sound = SA_UNEASE},
	{name = "You have no luck.", patch = "SA_NOLCK", sound = SA_UNEASE},
	{name = "RedXVI's house", patch = "SA_RDXVI", sound = SA_NORMAL},
	{name = "Smug titlescreen", patch = "SA_SMGTS", sound = SA_NAILEDIT},
	{name = "Sonic ???", patch = "SA_SON??", sound = SA_DOUBT},
	{name = "Tails ???", patch = "SA_TAL??", sound = SA_DOUBT},
	{name = "Spikes", patch = "SA_SPKES", sound = SA_UNEASE},
	{name = "uglyknux", patch = "SA_UGYKX"},
	{name = "you aren't safe", patch = "SA_UGYK2", sound = SA_ANGER},
	{name = "Walka", patch = "SA_WALKA", sound = SA_UNEASE},
	
	-- kart pack
	{name = "Kart Krew(tm)", patch = "SA_KTKRW", sound = SA_NAILEDIT},
	{name = "Banana meta", patch = "SA_BANNA", sound = SA_UNEASE},
	{name = "Cacodemon", patch = "SA_CDEMN", sound = SA_JOY},
	{name = "Cat Grimacing", patch = "SA_CTCRG", sound = SA_UNEASE},
	{name = "Really feeling it", patch = "SA_FLNIT", sound = SA_NAILEDIT},
	{name = "You fucking donkey", patch = "SA_FKDKY", sound = SA_ANGER},
	{name = "Kart E", patch = "SA_KARTE", sound = SA_NORMAL},
	{name = ":XD:", patch = "SA_KRTXD", sound = SA_JOY},
	{name = "Last laugh", patch = "SA_LLAGH", sound = SA_WHISTLE},
	{name = "Smug Lancer", patch = "SA_LNCER", sound = SA_DOUBT},
	{name = "Casually Messatsu 1st", patch = "SA_MESTU", sound = SA_UNEASE},
	{name = "Oh.", patch = "SA_OH", sound = SA_SORROW},
	{name = "Oh Snap", patch = "SA_OHSNP", sound = SA_WHISTLE},
	{name = "See you", patch = "SA_SEYOU", sound = SA_JOY},
	{name = "See you R2", patch = "SA_SEUR2", sound = SA_JOY},
	{name = "SPB with raised brow", patch = "SA_SPB", sound = SA_UNEASE},
	{name = "It's free real estate", patch = "SA_STFRE", sound = SA_NORMAL},
	{name = "Stop it.", patch = "SA_STPIT", sound = SA_UNEASE},
	{name = ":D?", patch = "SA_D?", sound = SA_HELP},
	{name = "In memory of", patch = "SA_TYL52", sound = SA_EMBARRASSMENT},
	{name = "You fucked up...", patch = "SA_UFKUP", sound = SA_UNEASE},
}
for k, v in ipairs(base_SAs) 
	table.insert(symbolArts, v) 
	print("Added Symbol Art "..#symbolArts..": "..v.name)
end

COM_AddCommand("sa_list", function(p, page)
	-- calculate how many pages we'll need
	local pagesize = 12
	local adjust = (#symbolArts%pagesize) and 1 or 0
	local pages = max(1, #symbolArts/pagesize + adjust)
	
	page = tonumber($)
	if not (page)
	or not (page >= 1 and page <= pages)
		CONS_Printf(p, "\x82WARNING:\x80 Enter a valid page! \x82(Total Pages: "..pages..")")
		return
	end
	
	CONS_Printf(p, "\x82".."Page "..page.."/"..pages..":")
	for i = pagesize*(page-1)+1, pagesize*page
		if not (symbolArts[i]) then continue end
		CONS_Printf(p, "\x82"..i..":\x80 "..symbolArts[i].name)
	end
end)