local function sa_sendSA(p, index)
	if not (server and server.valid) then return end
	
	if not (index) then return end
	if (p.sa_delay) then return end
	
	server.sa_playerSAs = $ or {}
	
	local info
	if (server.sa_playerSAs[#p])
		info = {p.name, p.skincolor, index, 5*TICRATE, server.sa_playerSAs[#p][5], server.sa_playerSAs[#p][6]}
	else
		-- make sure we dont get a position near the centre of the screen!
		local xlimit, ylimit = 70, 35
		local xpos, ypos = P_RandomRange(0, 270), P_RandomRange(0, 169)
		
		if (xpos >= xlimit and xpos <= 270 - xlimit)
		and (ypos >= ylimit and ypos <= 169 - (ylimit/2))
			-- snap them to either the xlimit or ylimit
			if (not P_RandomRange(0, 1))
				xpos = not P_RandomRange(0, 1) and P_RandomRange(0, xlimit) or P_RandomRange(270 - xlimit, 270)
			else
				ypos = not P_RandomRange(0, 1) and P_RandomRange(0, ylimit) or P_RandomRange(169 - (ylimit/2), 169)
			end
		end
				
		info = {p.name, p.skincolor, index, 5*TICRATE, xpos, ypos}
	end
	server.sa_playerSAs[#p] = info
	
	-- set delay
	p.sa_delay = cv_sa_delay.value
end

-- display symbol arts
COM_AddCommand("sa_display", function(p, index, ...)
	if not (server and server.valid) then return end
	
	if (p.sa_delay) then return end
	index = tonumber($)
	
	if not (index) then CONS_Printf(p, "\x82WARNING:\x80 No Symbol Art index given.") return end
	local sa = symbolArts[index]
	if not (sa) then CONS_Printf(p, "\x82WARNING:\x80 Symbol Art "..index.." not found.") return end
	sa_sendSA(p, index)
	
	-- check for tags
	local silent
	
	local args = {...}
	if (args)
		for k, v in pairs(args)
			if (v == "-silent") then silent = true continue end	-- silent detection
		end
	end
	
	if not (silent) then CONS_Printf(p, "Displaying Symbol Art "..index..": "..symbolArts[index].name) end
end)

local function sa_handleSAs()
	-- we'll use this to store what symbol arts are sent (by what players)
	server.sa_playerSAs = $ or {}
	if not (server.sa_playerSAs) then return end
	
	for k, v in pairs(server.sa_playerSAs)
		local sa = symbolArts[tonumber(v[3])]
		
		-- play sound
		if ((v[4] == 5*TICRATE) and (sa.sound))
		and (cv_sa.value and cv_sa_sounds.value)
			S_StartSound(nil, sa.sound)
		end
		
		-- decrease timer
		v[4] = $-1
		if not (v[4]) then server.sa_playerSAs[k] = nil end
	end
end

local function sa_handleDelays()
	for p in players.iterate
		if not (p.sa_delay) then continue end
		p.sa_delay = $-1
	end
end

addHook("ThinkFrame", do
	if not (server and server.valid) then return end
	
	sa_handleSAs()
	sa_handleDelays()
end)

local ascii_03d = {}
for i = 0, 255	-- generate ascii %03d table so we don't have to do string.format everytime, this function is very slow:
	ascii_03d[i] = string.format("%03d", tostring(i))
end

local function sa_hudFunction(v)
	if not (cv_sa.value) then return end
	
	if not (server and server.valid) then return end
	if not (server.sa_playerSAs) then return end
	
	local SAs = {}
	for index, info in pairs(server.sa_playerSAs)
		table.insert(SAs, info)
	end
	table.sort(SAs, function(a, b) return a[4] < b[4] end)
	
	for index, info in pairs(SAs)
		local scale = FRACUNIT/2
		local sa = symbolArts[tonumber(info[3])]
		
		-- draw bg + symbol art
		v.drawScaled(info[5]<<FRACBITS, info[6]<<FRACBITS, scale, v.cachePatch("SYMARTBG"), 0, v.getColormap(TC_DEFAULT, info[2]))
		v.drawScaled((info[5]+2)<<FRACBITS + (FRACUNIT/2), (info[6]+7)<<FRACBITS + (FRACUNIT/2), scale, v.cachePatch(sa.patch))
		
		-- player name
		if (info[1]:len())
			local hx = (info[5]+2)<<FRACBITS + (FRACUNIT/2)
			for i = 1, info[1]:len()
				local let = info[1]:sub(i, i)

				local p = "TNYFN"..ascii_03d[let:byte()]
				if v.patchExists(p)	-- draw letter
					local patch = v.cachePatch(p)	-- only cache once
					v.drawScaled(hx, (info[6]+2)<<FRACBITS, scale, patch)
					hx = $+((patch.width-1)*scale)		-- draw next letter inside the other a little.
				else	-- consider this a space
					hx = $+(3*scale)
				end
			end
		end
	end
end

-- there is no escape from the symbol arts
local hudHooks = {"game", "scores", "title", "titlecard", "intermission"}
for i = 1, #hudHooks do hud.add(sa_hudFunction, hudHooks[i]) end